<?php
/**
 * This is the datadict controller class.
 * @author Cheng Gang <chenggang.china@gmail.com>
 * @link http://www.chenggang.net/
 * @copyright Copyright &copy; 2009-2010 chenggang
 * @date time 2010-11-28 12:35:00 */

class DatadictController extends Controller
{
	/**
	 * @var CActiveRecord 当前载入的model实例
	 */
	private $_model;

	/**
	 * @return array action 过滤器
	 */
	public function filters()
	{
		return array(
			'accessControl', // perform access control for CRUD operations
		);
	}

	/**
	 * 制定访问控制规则
	 * 使用这个方法的是'accessControl'过滤器.
	 * @return array 访问控制规则
	 */
	public function accessRules()
	{
		return array(
			array('allow',  // 用户所有用户执行的操作
				'actions'=>array('index','view','add','update','delete','mdelete','activate','search','datadict','updatedict','mactivate'),
				'users'=>array('@'),
			),
			/*array('allow', // 允许所有认证的用户执行的操作
				'actions'=>array('create','update'),
				'users'=>array('@'),
			),
			array('allow', // 允许admin这个用户执行的操作
				'actions'=>array('admin','delete'),
				'users'=>array('admin'),
			),*/
			array('deny',  // 其他操作拒绝所有用户访问
				'users'=>array('*'),
			),
		);
	}

	/**
	 * 显示model单个记录
	 */
	public function actionView()
	{
		$this->_view['model']=$this->loadModel();
		$this->render('view');
	}

	/**
	 * 创建新model记录， 如果创建成功默认重定向到view页
	 */
	public function actionAdd($tid=0)
	{
		$model=new DataDict;
		$model->table_id = $tid;

		// 如果需要AJAX验证请取消下面这一行的注释
		// $this->performAjaxValidation($model);

		if(isset($_POST['DataDict']))
		{
			$model->attributes=$_POST['DataDict'];
			$model->language = Yii::app()->language;
			$model->status   = 1;
												
			if($model->save())
			{
			    if(Yii::app()->request->isAjaxRequest)
				    $this->sendOk(Yii::t('models/table','Add Success'), $this->viewForward($model->id));
				else
				    $this->redirect(array('view','id'=>$model->id));
			}
			elseif(Yii::app()->request->isAjaxRequest)
				$this->sendError($model);
		}

		$this->_view['model']=$model;
		$this->render('add');
	}

	/**
	 * 更新指定的model记录.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionUpdate()
	{
		$model=$this->loadModel();

		// 如果需要AJAX验证请取消下面这一行的注释
		// $this->performAjaxValidation($model);

		if(isset($_POST['DataDict']))
		{
			$model->attributes=$_POST['DataDict'];
			if($model->save())
			{
			    if(Yii::app()->request->isAjaxRequest)
				    $this->sendOk(Yii::t('models/table','Update Success'), $this->viewForward($model->id));
				else
				    $this->redirect(array('view','id'=>$model->id));
			}
			elseif(Yii::app()->request->isAjaxRequest)
				$this->sendError($model);
		}

		$this->_view['model']=$model;
		$this->render('update');
	}

	/**
	 * 删除指定的model记录，如果删除成功则默认重定向到index页
	 */
	public function actionDelete()
	{
		if(Yii::app()->request->isPostRequest)
		{
			// 删除操作只允许POST请求操作
			$this->loadModel()->delete();

			// 这里如果是AJAX请求（例如管理员通过grid view请求的删除操作），就不要重定向。
			if(!isset($_GET['ajax']))
				$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
		}
		else
			throw new CHttpException(400,Yii::t('public','Invalid request. Please do not repeat this request again.'));
	}

	/**
	 * 列出model所有记录
	 */
	public function actionIndex($tid=0)
	{
		$criteria = new CDbCriteria();
		$criteria->select 		= " t.* ";
		$criteria->compare('language',Yii::app()->language);
		
		if(isset($_GET['DataDict'])) 
		{			
			$attributes=$_GET['DataDict'];			
						
			if(isset($attributes['table_id']) && $attributes['table_id']!='') $criteria->compare('table_id',$attributes['table_id']);						
			if(isset($attributes['field'])) $criteria->compare('field',$attributes['field'],true);						
			if(isset($attributes['tips'])) $criteria->compare('tips',$attributes['tips'],true);						
			if(isset($attributes['name'])) $criteria->compare('name',$attributes['name'],true);						
			if(isset($attributes['data_type'])) $criteria->compare('data_type',$attributes['data_type'],true);						
			if(isset($attributes['length'])) $criteria->compare('length',$attributes['length'],true);						
			if(isset($attributes['description'])) $criteria->compare('description',$attributes['description'],true);						
			//if(isset($attributes['language'])) $criteria->compare('language',$attributes['language'],true);								
		}
		else
		{
		    $criteria->compare('table_id',$tid);
		}
	
		$dataProvider=new CActiveDataProvider('DataDict', array(
		    'criteria'=>$criteria,
		    'pagination'=>array(
		    	'pageSize'=>Yii::app()->params['pageSize'],
		    ),
		));
		$this->_view['dataProvider']=$dataProvider;
		$this->_view['table']=DataDict::model()->findByPk($tid);
		$this->render('index');
	}
	
    //生成数据字典
	public function actionDatadict()
	{
	    $num = 0;
	    //TRUNCATE TABLE `data_dict` 
	    $tables=Yii::app()->db->schema->getTables();
	    foreach($tables AS $table)
	    {
	        $attr = array(
						'data_type'=>'TABLE',
						'language'=>Yii::app()->language,
	                    'field'=>$table->name,
	                    'status'=>1
                        );
	        $model=DataDict::model()->findByAttributes($attr);
	        if($model===null)
		    {
		        $model = new DataDict;
    	        $model->status = 1;
    	        $model->data_type = 'TABLE';
    	        $model->language = Yii::app()->language;
    	        $model->field = $table->name;
    	        $model->name  = $table->name;
    	        $num +=$model->save() ? 1 : 0;
		    }	        
	        
	        foreach($table->columns AS $column)
	        {
	            $type = $column->dbType;
	            $type = explode('(', $type);
	            $data_type = $type[0];
	            $data_type = (strpos($data_type,'int')!==false)? 'INT':strtoupper($data_type);
	            
	            $attr = array(
	                    'table_id'=>$model->id,
						'data_type'=>$data_type,
						'language'=>Yii::app()->language,	                    
	                    'field'=>$column->name,
	                    'status'=>1
                        );
    	        $fmodel=DataDict::model()->findByAttributes($attr);
    	        if($fmodel===null)
    		    {	            
    	            $fmodel = new DataDict;
    	            $fmodel->status   = 1;
    	            $fmodel->table_id = $model->id;
        	        $fmodel->data_type= $data_type;
        	        $fmodel->language = Yii::app()->language;
        	        $fmodel->field    = $column->name;
        	        $fmodel->name     = $column->name;
        	        $fmodel->length   = $column->size;
        	        if(!$fmodel->save())
        	        {
        	            /*echo $fmodel->data_type;
        	            print_r($fmodel->errors);
        	            print_r($column);
        	            exit;*/
        	            $this->sendError($fmodel);
        	        } 
        	        else
        	        {
        	            $num++;
        	        }
    		    }
	        }
	    }
	    //echo 'OK';
	    //Yii::app()->end();
	    $this->sendOk(Yii::t('models/table', "Total {num} item(s) add.",array('{num}' => $num)));
	}
	
    //更新数据字典
	public function actionUpdatedict()
	{
	    $num = 0;
	    $models = DataDict::model()->findAll("language='".Yii::app()->language."'");
	    foreach($models AS $model)
	    {
	        if($model->data_type=='TABLE')
	        {
	            if(!Yii::app()->db->schema->getTable($model->field))
	            {
	                if($model->rdelete() && $model->deleteFile($model->field)) $num++;
	            }
	        }
	        else 
	        {
	            $table = is_object($model->parent) ? Yii::app()->db->schema->getTable($model->parent->field) : '';
	            $flag  = 1;
	            if(!is_object($table) && $model->rdelete())
	            { 
	                $num++;
	            }
	            else 
	            {
    	            foreach($table->columns AS $column)
    	            {
    	                if($model->field == $column->name)
    	                {
    	                    $flag = 0;
    	                    break;
    	                }
    	            }
    	            if($flag && $model->deleteMessage($model->parent->field, $model->field)) 
    	            {
    	                $num++;
    	            }
    	            else 
    	            {
    	                $type = $column->dbType;
        	            $type = explode('(', $type);
        	            $data_type = $type[0];
        	            $data_type = (strpos($data_type,'int')!==false)? 'INT':strtoupper($data_type);
            	        $model->data_type = $data_type;
            	        $model->length    = $column->size;
            	        if(!$model->save())
            	        {
            	            /*echo $model->data_type;
            	            print_r($model->errors);
            	            print_r($column);
            	            exit;*/
            	            $this->sendError($model);
            	        } 
            	        else
            	        {
            	            $num++;
            	        }
    	            }
	            }
	        }
	    }
	    	    
	    $this->sendOk(Yii::t('models/table', "Total {num} item(s) update.",array('{num}' => $num)));
	}

	/**
	 * 根据GET变量返回表的主键记录,如果没有找到则抛出错误
	 */
	public function loadModel()
	{
		if($this->_model===null)
		{
			if(isset($_GET['id']))
				$this->_model=DataDict::model()->findbyPk($_GET['id']);
			if($this->_model===null)
				throw new CHttpException(404,Yii::t('public','The requested page does not exist.'));
		}
		return $this->_model;
	}

	/**
	 * 执行AJAX验证
	 * @param CModel 被验证的model
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='data_dict-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
	
	/**
	* 批量删除
	**/
	public function actionMdelete()
    {
		if(isset($_POST['check_data_dict']))
		{
			$ids = $_POST['check_data_dict'];

			if(empty($ids))
			{
				$this->sendWarn(Yii::t('models/table','Please select the item'));
			}
			else
			{
			    $num = 0;
				//这种处理请把处理部分写在model
				foreach($ids AS $id)
				{
					$model = DataDict::model()->findbyPk($id);
                    if($model->delete()) $num++;
				}
				$this->sendOk(Yii::t('models/table',"Total {num} item(s) delete.",array('{num}'=>$num)));
			}
		}
		else
			$this->sendError(Yii::t('models/table','Please select the item'));
			//throw new CHttpException(400,Yii::t('public','Invalid request. Please do not repeat this request again.'));
    }
    
	/**
	* 批量激活
	**/
	public function actionMactivate()
    {
		if(isset($_POST['check_data_dict']))
		{
			$ids = $_POST['check_data_dict'];

			if(empty($ids))
			{
				$this->sendWarn(Yii::t('models/table','Please select the item'));
			}
			else
			{
			    $num = 0;
				//这种处理请把处理部分写在model
				foreach($ids AS $id)
				{
					$model = DataDict::model()->findbyPk($id);
                    if($model->activate()) $num++;
				}
				$this->sendOk(Yii::t('models/table',"Total {num} item(s) activate.",array('{num}'=>$num)));
			}
		}
		else
			$this->sendError(Yii::t('models/table','Please select the item'));
			//throw new CHttpException(400,Yii::t('public','Invalid request. Please do not repeat this request again.'));
    }
}
