<?php
/**
 * This is the model class for table "data_dict".
 * @author Cheng Gang <chenggang.china@gmail.com>
 * @link http://www.chenggang.net/
 * @copyright Copyright &copy; 2009-2010 chenggang
 * @date time 2010-11-24 15:31:19 */
class DataDict extends ActiveRecord
{
	/**
	 * The followings are the available columns in table 'data_dict':
	 * @var integer $id
	 * @var string $field
	 * @var string $tips
	 * @var string $name
	 * @var string $data_type
	 * @var string $length
	 * @var string $description
	 * @var integer $status
	 */

	/**
	 * Returns the static model of the specified AR class.
	 * @return DataDict the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{data_dict}}';
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Yii::t('models/DataDict','ID'),
		    'table_id' => Yii::t('models/DataDict','Table'),
			'field' => Yii::t('models/DataDict','Field'),
			'tips' => Yii::t('models/DataDict','Tips'),
			'name' => Yii::t('models/DataDict','Name'),
			'data_type' => Yii::t('models/DataDict','Data Type'),
			'length' => Yii::t('models/DataDict','Length'),
			'description' => Yii::t('models/DataDict','Description'),
			'status' => Yii::t('models/DataDict','Status'),
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('field',$this->field,true);
		$criteria->compare('tips',$this->tips,true);
		$criteria->compare('name',$this->name,true);
		$criteria->compare('data_type',$this->data_type,true);
		$criteria->compare('length',$this->length,true);
		$criteria->compare('description',$this->description,true);
		return new CActiveDataProvider(get_class($this), array(
			'criteria'=>$criteria,
		));
	}
	
	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('field, name, data_type', 'required'),
			array('status', 'numerical', 'integerOnly'=>true),
			array('field', 'length', 'max'=>64),
			array('tips', 'length', 'max'=>255),
			array('name, description', 'length', 'max'=>255),
			array('table_id,data_type', 'length', 'max'=>10),
			array('length', 'length', 'max'=>50),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, field, tips, name, data_type, length, description, status', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'parent' => array(self::BELONGS_TO, 'DataDict', 'table_id','select'=>'table_id,field,name'),
		);
	}
	
    protected function afterSave()
    {
        parent::afterSave();
        $this->array2file();
    }
	
	//将数据字典写入翻译文件中
	public function array2file()
	{
	    if($this->data_type=='TABLE')
	    {
	        $tableName = $this->field;
	        $className = $this->generateClassName($tableName);
	        $label     = $this->class2label($className);
	        $label     = $this->pluralize($label);
	    }
	    else
	    {
	        $tableName = $this->parent->field;
	        $className = $this->generateClassName($tableName);
	        $label     = $this->generateLabels($this->field);
	    }
	    $path = $this->messageFile($className);
	    $messages  = $this->getMessage($path);
	    $messages[$label] = $this->name; 
	    //$messages[$label] = $label; 
	    if($this->tips)
	    {
	        $messages{'Tips '.$label} = $this->tips; 
	    }
	    
	    return $this->writeMessage($path, $messages);
	}
	
	public function class2label($modelClass)
	{
	    return trim(str_replace('-',' ',preg_replace('/(?<![A-Z])[A-Z]/', '-\0', $modelClass)));
	}
	
    public function generateLabels($field)
	{
		$label=ucwords(trim(strtolower(str_replace(array('-','_'),' ',preg_replace('/(?<![A-Z])[A-Z]/', ' \0', $field)))));
		$label=preg_replace('/\s+/',' ',$label);
		if(strcasecmp(substr($label,-3),' id')===0)
			$label=substr($label,0,-3);
		if($label==='Id')
			$label='ID';
			
		return $label;
	}
	
	public function messageFile($modelClass)
	{
	    return Yii::app()->basePath.DIRECTORY_SEPARATOR.'messages'.DIRECTORY_SEPARATOR.Yii::app()->language.DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.$modelClass.'.php';
	}
	
	public function getMessage($path)
	{
	    if(is_file($path))
	    {
	        $messages=require($path);
	    }
	    else
	    {
	        $messages=array();
	    }
	    return $messages;
	}	
	
    protected function removePrefix($tableName,$addBrackets=true)
	{
		if($addBrackets && Yii::app()->db->tablePrefix=='')
			return $tableName;
		
		$prefix=Yii::app()->db->tablePrefix;
		if($prefix!='')
		{
			if($addBrackets && Yii::app()->db->tablePrefix!='')
			{
				$prefix=Yii::app()->db->tablePrefix;
				$lb='{{';
				$rb='}}';
			}
			else
				$lb=$rb='';
			if(($pos=strrpos($tableName,'.'))!==false)
			{
				$schema=substr($tableName,0,$pos);
				$name=substr($tableName,$pos+1);
				if(strpos($name,$prefix)===0)
					return $schema.'.'.$lb.substr($name,strlen($prefix)).$rb;
			}
			else if(strpos($tableName,$prefix)===0)
				return $lb.substr($tableName,strlen($prefix)).$rb;
		}
		return $tableName;
	}
	
    public function pluralize($name)
	{
		$rules=array(
			'/(x|ch|ss|sh|us|as|is|os)$/i' => '\1es',
			'/(?:([^f])fe|([lr])f)$/i' => '\1\2ves',
			'/(m)an$/i' => '\1en',
			'/(child)$/i' => '\1ren',
			'/(r|t)y$/i' => '\1ies',
			'/s$/' => 's',
		);
		foreach($rules as $rule=>$replacement)
		{
			if(preg_match($rule,$name))
				return preg_replace($rule,$replacement,$name);
		}
		return $name.'s';
	}
	
    protected function generateClassName($tableName)
	{
		$tableName=$this->removePrefix($tableName,false);
		$className='';
		foreach(explode('_',$tableName) as $name)
		{
			if($name!=='')
				$className.=ucfirst($name);
		}
		return $className;
	}
	
	public function writeMessage($path,$messages)
	{
	    $content = "<?php \nreturn array(\n";
	    foreach($messages AS $key=>$message)
	    {
	        $content .= sprintf("'%s'=>'%s',\n",$key,$message);
	    }
	    $content .= ");";
	    if ($handle = fopen($path, 'w'))
		{
			fwrite($handle, $content);
			fclose($handle);
		}
		
		return TRUE;
	}
	
	public function deleteFile($tableName)
	{
	    $className = $this->generateClassName($tableName);
	    $path = $this->messageFile($className);
	    
	    return is_file($path) ? unlink($path) : true;
	}
	
	public function deleteMessage($tableName,$field)
	{
        $className = $this->generateClassName($tableName);
        $label     = $this->generateLabels($this->field);
	    $path      = $this->messageFile($className);
	    $messages  = $this->getMessage($path);
	    unset($messages[$label]);
	    unset($messages{'Tips '.$label});
	    
	    return $this->writeMessage($path, $messages);
	}
	
	
	
	
	
}